@@ -10,31 +10,33 @@ Without him this would not have been possible.
 from __future__ import annotations
 
 import abc
-from dataclasses import dataclass
 import errno
 import json
 import logging
-import select
+import selectors
 import socket
 import ssl
 import threading
 import time
 from collections import defaultdict
+from dataclasses import dataclass
 from struct import pack, unpack
 
 import zeroconf
 
-from .controllers import CallbackType, BaseController
+from .config import APP_AUDIBLE
+from .const import MESSAGE_TYPE, PLATFORM_DESTINATION_ID, REQUEST_ID, SESSION_ID
+from .controllers import BaseController, CallbackType
+from .controllers.heartbeat import NS_HEARTBEAT, HeartbeatController
 from .controllers.media import MediaController
 from .controllers.receiver import CastStatus, CastStatusListener, ReceiverController
-from .const import MESSAGE_TYPE, REQUEST_ID, SESSION_ID
 from .dial import get_host_from_service
 from .error import (
     ChromecastConnectionError,
     ControllerNotRegistered,
-    UnsupportedNamespace,
     NotConnected,
     PyChromecastStopped,
+    UnsupportedNamespace,
 )
 
 # pylint: disable-next=no-name-in-module
@@ -42,12 +44,7 @@ from .generated.cast_channel_pb2 import CastMessage
 from .models import HostServiceInfo, MDNSServiceInfo
 
 NS_CONNECTION = "urn:x-cast:com.google.cast.tp.connection"
-NS_HEARTBEAT = "urn:x-cast:com.google.cast.tp.heartbeat"
 
-PLATFORM_DESTINATION_ID = "receiver-0"
-
-TYPE_PING = "PING"
-TYPE_PONG = "PONG"
 TYPE_CONNECT = "CONNECT"
 TYPE_CLOSE = "CLOSE"
 TYPE_LOAD = "LOAD"
@@ -64,13 +61,8 @@ CONNECTION_STATUS_FAILED = "FAILED"
 CONNECTION_STATUS_FAILED_RESOLVE = "FAILED_RESOLVE"
 # The socket connection was lost and needs to be retried
 CONNECTION_STATUS_LOST = "LOST"
-# Check for select poll method
-SELECT_HAS_POLL = hasattr(select, "poll")
 
-HB_PING_TIME = 10
-HB_PONG_TIME = 10
-POLL_TIME_BLOCKING = 5.0
-POLL_TIME_NON_BLOCKING = 0.01
+SELECT_TIMEOUT = 5.0
 TIMEOUT_TIME = 30.0
 RETRY_TIME = 5.0
 
@@ -215,6 +207,11 @@ class SocketClient(threading.Thread, CastStatusListener):
         self.connecting = True
         self.first_connection = True
         self.socket: socket.socket | ssl.SSLSocket | None = None
+        self.selector = selectors.DefaultSelector()
+        self.wakeup_selector_key = self.selector.register(
+            self.socketpair[0], selectors.EVENT_READ
+        )
+        self.remote_selector_key: selectors.SelectorKey | None = None
 
         # dict mapping namespace on Controller objects
         self._handlers: dict[str, set[BaseController]] = defaultdict(set)
@@ -238,8 +235,10 @@ class SocketClient(threading.Thread, CastStatusListener):
         tries = self.tries
 
         if self.socket is not None:
+            self.selector.unregister(self.socket)
             self.socket.close()
             self.socket = None
+            self.remote_selector_key = None
 
         # Make sure nobody is blocking.
         for callback_function in self._request_callbacks.values():
@@ -288,10 +287,15 @@ class SocketClient(threading.Thread, CastStatusListener):
                 try:
                     if self.socket is not None:
                         # If we retry connecting, we need to clean up the socket again
-                        self.socket.close()  # type: ignore[unreachable]
+                        self.selector.unregister(self.socket)  # type: ignore[unreachable]
+                        self.socket.close()
                         self.socket = None
+                        self.remote_selector_key = None
 
                     self.socket = new_socket()
+                    self.remote_selector_key = self.selector.register(
+                        self.socket, selectors.EVENT_READ
+                    )
                     self.socket.settimeout(self.timeout)
                     self._report_connection_status(
                         ConnectionStatus(
@@ -482,6 +486,17 @@ class SocketClient(threading.Thread, CastStatusListener):
         self.session_id = status.session_id
 
         if new_channel and self.destination_id is not None:
+            # App quirk: If the Audible app is running, wait 1s after receiving cast status
+            # before connecting to the media player channel. This is a workaround for
+            # https://github.com/home-assistant-libs/pychromecast/issues/738
+            if (
+                any(namespace in self._handlers for namespace in self.app_namespaces)
+                and self.destination_id not in self._open_channels
+                and status.app_id == APP_AUDIBLE
+            ):
+                self.logger.debug("Detected Audible connection. Sleeping for 1s")
+                time.sleep(1)
+
             # If any of the namespaces of the new app are supported
             # we will automatically connect to it to receive updates
             for namespace in self.app_namespaces:
@@ -530,7 +545,7 @@ class SocketClient(threading.Thread, CastStatusListener):
         self.logger.debug("Thread started...")
         while not self.stop.is_set():
             try:
-                if self._run_once(timeout=POLL_TIME_BLOCKING) == 1:
+                if self._run_once() == 1:
                     break
             except Exception:  # pylint: disable=broad-except
                 self._force_recon = True
@@ -545,7 +560,7 @@ class SocketClient(threading.Thread, CastStatusListener):
         # Clean up
         self._cleanup()
 
-    def _run_once(self, timeout: float = POLL_TIME_NON_BLOCKING) -> int:
+    def _run_once(self) -> int:
         """Receive from the socket and handle data."""
         # pylint: disable=too-many-branches, too-many-statements, too-many-return-statements
 
@@ -558,21 +573,13 @@ class SocketClient(threading.Thread, CastStatusListener):
         # A connection has been established at this point by self._check_connection
         assert self.socket is not None
 
-        # poll the socket, as well as the socketpair to allow us to be interrupted
-        rlist = [self.socket, self.socketpair[0]]
+        # Poll the socket and the socketpair, with a timeout of SELECT_TIMEOUT
+        # The timeout ensures we call _check_connection often enough to avoid
+        # the HeartbeatController from detecting a timeout
+        # The socketpair allow us to be interrupted on shutdown without waiting
+        # for the SELECT_TIMEOUT timout to expire
         try:
-            if SELECT_HAS_POLL is True:
-                # Map file descriptors to socket objects because select.select does not support fd > 1024
-                # https://stackoverflow.com/questions/14250751/how-to-increase-filedescriptors-range-in-python-select
-                fd_to_socket = {rlist_item.fileno(): rlist_item for rlist_item in rlist}
-
-                poll_obj = select.poll()
-                for poll_fd in rlist:
-                    poll_obj.register(poll_fd, select.POLLIN)
-                poll_result = poll_obj.poll(timeout * 1000)  # timeout in milliseconds
-                can_read = [fd_to_socket[fd] for fd, _status in poll_result]
-            else:
-                can_read, _, _ = select.select(rlist, [], [], timeout)
+            ready = self.selector.select(SELECT_TIMEOUT)
         except (ValueError, OSError) as exc:
             self.logger.error(
                 "[%s(%s):%s] Error in select call: %s",
@@ -584,9 +591,10 @@ class SocketClient(threading.Thread, CastStatusListener):
             self._force_recon = True
             return 0
 
+        can_read = {key for key, _ in ready}
         # read message from chromecast
         message = None
-        if self.socket in can_read and not self._force_recon:
+        if self.remote_selector_key in can_read and not self._force_recon:
             try:
                 message = self._read_message()
             except InterruptLoop as exc:
@@ -611,18 +619,17 @@ class SocketClient(threading.Thread, CastStatusListener):
                     if self.stop.is_set():
                         return 1
                 raise
-            except socket.error:
+            except socket.error as exc:
                 self._force_recon = True
                 self.logger.error(
-                    "[%s(%s):%s] Error reading from socket.",
+                    "[%s(%s):%s] Error reading from socket: %s",
                     self.fn or "",
                     self.host,
                     self.port,
+                    exc,
                 )
-            else:
-                data = _dict_from_message_payload(message)
 
-        if self.socketpair[0] in can_read:
+        if self.wakeup_selector_key in can_read:
             # Clear the socket's buffer
             self.socketpair[0].recv(128)
 
@@ -635,6 +642,7 @@ class SocketClient(threading.Thread, CastStatusListener):
             return 0
 
         # See if any handlers will accept this message
+        data = _dict_from_message_payload(message)
         self._route_message(message, data)
 
         if REQUEST_ID in data and data[REQUEST_ID] in self._request_callbacks:
@@ -652,8 +660,8 @@ class SocketClient(threading.Thread, CastStatusListener):
         # check if connection is expired
         reset = False
         if self._force_recon:
-            self.logger.warning(
-                "[%s(%s):%s] Error communicating with socket, resetting connection",
+            self.logger.debug(
+                "[%s(%s):%s] Forced reconnection",
                 self.fn or "",
                 self.host,
                 self.port,
@@ -661,7 +669,7 @@ class SocketClient(threading.Thread, CastStatusListener):
             reset = True
 
         elif self.heartbeat_controller.is_expired():
-            self.logger.warning(
+            self.logger.info(
                 "[%s(%s):%s] Heartbeat timeout, resetting connection",
                 self.fn or "",
                 self.host,
@@ -690,7 +698,10 @@ class SocketClient(threading.Thread, CastStatusListener):
         # route message to handlers
         if message.namespace in self._handlers:
             # debug messages
-            if message.namespace != NS_HEARTBEAT:
+            if (
+                message.namespace != NS_HEARTBEAT
+                or self.heartbeat_controller.logger.isEnabledFor(logging.DEBUG)
+            ):
                 self.logger.debug(
                     "[%s(%s):%s] Received: %s",
                     self.fn or "",
@@ -767,6 +778,7 @@ class SocketClient(threading.Thread, CastStatusListener):
 
         self.socketpair[0].close()
         self.socketpair[1].close()
+        self.selector.close()
 
         self.connecting = True
 
@@ -873,7 +885,10 @@ class SocketClient(threading.Thread, CastStatusListener):
         be_size = pack(">I", msg.ByteSize())
 
         # Log all messages except heartbeat
-        if msg.namespace != NS_HEARTBEAT:
+        if (
+            msg.namespace != NS_HEARTBEAT
+            or self.heartbeat_controller.logger.isEnabledFor(logging.DEBUG)
+        ):
             self.logger.debug(
                 "[%s(%s):%s] Sending: %s",
                 self.fn or "",
@@ -897,17 +912,18 @@ class SocketClient(threading.Thread, CastStatusListener):
                     else:
                         callback_function(True, None)
                 self.socket.sendall(be_size + msg.SerializeToString())
-            except socket.error:
+            except socket.error as exc:
                 if callback_function:
                     callback_function(False, None)
                 if not no_add_request_id:
                     self._request_callbacks.pop(request_id, None)
                 self._force_recon = True
-                self.logger.info(
-                    "[%s(%s):%s] Error writing to socket.",
+                self.logger.warning(
+                    "[%s(%s):%s] Error writing to socket: %s",
                     self.fn or "",
                     self.host,
                     self.port,
+                    exc,
                 )
         else:
             if callback_function:
@@ -1061,73 +1077,6 @@ class ConnectionController(BaseController):
         return False
 
 
-class HeartbeatController(BaseController):
-    """Controller to respond to heartbeat messages."""
-
-    def __init__(self) -> None:
-        super().__init__(NS_HEARTBEAT, target_platform=True)
-        self.last_ping = 0.0
-        self.last_pong = time.time()
-
-    def receive_message(self, _message: CastMessage, data: dict) -> bool:
-        """
-        Called when a heartbeat message is received.
-
-        data is message.payload_utf8 interpreted as a JSON dict.
-        """
-        if self._socket_client is None:
-            raise ControllerNotRegistered
-
-        if self._socket_client.is_stopped:
-            return True
-
-        if data[MESSAGE_TYPE] == TYPE_PING:
-            try:
-                self._socket_client.send_message(
-                    PLATFORM_DESTINATION_ID,
-                    self.namespace,
-                    {MESSAGE_TYPE: TYPE_PONG},
-                    no_add_request_id=True,
-                )
-            except PyChromecastStopped:
-                self._socket_client.logger.debug(
-                    "Heartbeat error when sending response, "
-                    "Chromecast connection has stopped"
-                )
-
-            return True
-
-        if data[MESSAGE_TYPE] == TYPE_PONG:
-            self.reset()
-            return True
-
-        return False
-
-    def ping(self) -> None:
-        """Send a ping message."""
-        if self._socket_client is None:
-            raise ControllerNotRegistered
-
-        self.last_ping = time.time()
-        try:
-            self.send_message({MESSAGE_TYPE: TYPE_PING})
-        except NotConnected:
-            self._socket_client.logger.error(
-                "Chromecast is disconnected. Cannot ping until reconnected."
-            )
-
-    def reset(self) -> None:
-        """Reset expired counter."""
-        self.last_pong = time.time()
-
-    def is_expired(self) -> bool:
-        """Indicates if connection has expired."""
-        if time.time() - self.last_ping > HB_PING_TIME:
-            self.ping()
-
-        return (time.time() - self.last_pong) > HB_PING_TIME + HB_PONG_TIME
-
-
 def new_socket() -> socket.socket:
     """
     Create a new socket with OS-specific parameters
