@@ -1,23 +1,23 @@
 """
 Implements the DIAL-protocol to communicate with the Chromecast
 """
-from collections import namedtuple
+
+from __future__ import annotations
+
+from dataclasses import dataclass
 import json
 import logging
 import socket
 import ssl
 import urllib.request
 from uuid import UUID
+from typing import Any
 
 import zeroconf
 
-from .const import (
-    CAST_TYPE_AUDIO,
-    CAST_TYPE_CHROMECAST,
-    CAST_TYPE_GROUP,
-    SERVICE_TYPE_HOST,
-)
-from .models import ZEROCONF_ERRORS, CastInfo, ServiceInfo
+from .const import CAST_TYPE_AUDIO, CAST_TYPE_CHROMECAST, CAST_TYPE_GROUP
+from .error import ZeroConfInstanceRequired
+from .models import ZEROCONF_ERRORS, CastInfo, HostServiceInfo, MDNSServiceInfo
 
 XML_NS_UPNP_DEVICE = "{urn:schemas-upnp-org:device-1-0}"
 
@@ -27,15 +27,19 @@ FORMAT_BASE_URL_HTTPS = "https://{}:8443"
 _LOGGER = logging.getLogger(__name__)
 
 
-def get_host_from_service(service, zconf):
+def get_host_from_service(
+    service: HostServiceInfo | MDNSServiceInfo, zconf: zeroconf.Zeroconf | None
+) -> tuple[str | None, int | None, zeroconf.ServiceInfo | None]:
     """Resolve host and port from service."""
     service_info = None
 
-    if service.type == SERVICE_TYPE_HOST:
-        return service.data + (None,)
+    if isinstance(service, HostServiceInfo):
+        return (service.host, service.port, None)
 
     try:
-        service_info = zconf.get_service_info("_googlecast._tcp.local.", service.data)
+        if not zconf:
+            raise ZeroConfInstanceRequired
+        service_info = zconf.get_service_info("_googlecast._tcp.local.", service.name)
         if service_info:
             _LOGGER.debug(
                 "get_info_from_service resolved service %s to service_info %s",
@@ -54,24 +58,30 @@ def get_host_from_service(service, zconf):
     return _get_host_from_zc_service_info(service_info) + (service_info,)
 
 
-def _get_host_from_zc_service_info(service_info: zeroconf.ServiceInfo):
+def _get_host_from_zc_service_info(
+    service_info: zeroconf.ServiceInfo | None,
+) -> tuple[str | None, int | None]:
     """Get hostname or IP + port from zeroconf service_info."""
     host = None
     port = None
-    if (
-        service_info
-        and service_info.port
-        and (service_info.server or len(service_info.addresses) > 0)
-    ):
+    if service_info and service_info.port:
         if len(service_info.addresses) > 0:
             host = socket.inet_ntoa(service_info.addresses[0])
-        else:
+        elif service_info.server is not None:
             host = service_info.server.lower()
-        port = service_info.port
+        if host is not None:
+            port = service_info.port
     return (host, port)
 
 
-def _get_status(services, zconf, path, secure, timeout, context):
+def _get_status(
+    services: set[HostServiceInfo | MDNSServiceInfo],
+    zconf: zeroconf.Zeroconf | None,
+    path: str,
+    secure: bool,
+    timeout: float,
+    context: ssl.SSLContext | None,
+) -> tuple[str | None, Any]:
     """Query a cast device via http(s)."""
 
     for service in services.copy():
@@ -97,26 +107,28 @@ def _get_status(services, zconf, path, secure, timeout, context):
     return (host, json.loads(data.decode("utf-8")))
 
 
-def get_ssl_context():
+def get_ssl_context() -> ssl.SSLContext:
     """Create an SSL context."""
-    context = ssl.SSLContext()
+    context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
+    context.check_hostname = False
     context.verify_mode = ssl.CERT_NONE
     return context
 
 
-def get_cast_type(cast_info, zconf=None, timeout=30, context=None):
-    """
-    :param cast_info: cast_info
-    :return: An updated cast_info with filled cast_type
-    :rtype: pychromecast.models.CastInfo
-    """
+def get_cast_type(
+    cast_info: CastInfo,
+    zconf: zeroconf.Zeroconf | None = None,
+    timeout: float = 30,
+    context: ssl.SSLContext | None = None,
+) -> CastInfo:
+    """Add cast type and manufacturer to a CastInfo instance."""
     cast_type = CAST_TYPE_CHROMECAST
     manufacturer = "Unknown manufacturer"
     if cast_info.port != 8009:
         cast_type = CAST_TYPE_GROUP
         manufacturer = "Google Inc."
     else:
-        host = "<unknown>"
+        host: str | None = "<unknown>"
         try:
             display_supported = True
             host, status = _get_status(
@@ -165,26 +177,37 @@ def get_cast_type(cast_info, zconf=None, timeout=30, context=None):
 
 
 def get_device_info(  # pylint: disable=too-many-locals
-    host, services=None, zconf=None, timeout=30, context=None
-):
-    """
-    :param host: Hostname or ip to fetch status from
-    :type host: str
-    :return: The device status as a named tuple.
-    :rtype: pychromecast.dial.DeviceStatus or None
-    """
+    host: str,
+    services: set[HostServiceInfo | MDNSServiceInfo] | None = None,
+    zconf: zeroconf.Zeroconf | None = None,
+    timeout: float = 30,
+    context: ssl.SSLContext | None = None,
+) -> DeviceStatus | None:
+    """Return a filled in DeviceStatus object for the specified device."""
 
     try:
         if services is None:
-            services = [ServiceInfo(SERVICE_TYPE_HOST, (host, 8009))]
-        _, status = _get_status(
-            services,
-            zconf,
-            "/setup/eureka_info?params=device_info,name",
-            True,
-            timeout,
-            context,
-        )
+            services = {HostServiceInfo(host, 8009)}
+
+        # Try connection with SSL first, and if it fails fall back to non-SSL
+        try:
+            _, status = _get_status(
+                services,
+                zconf,
+                "/setup/eureka_info?params=device_info,name",
+                True,
+                timeout / 2,
+                context,
+            )
+        except (urllib.error.HTTPError, urllib.error.URLError):
+            _, status = _get_status(
+                services,
+                zconf,
+                "/setup/eureka_info?params=device_info,name",
+                False,
+                timeout / 2,
+                context,
+            )
 
         cast_type = CAST_TYPE_CHROMECAST
         display_supported = True
@@ -204,6 +227,8 @@ def get_device_info(  # pylint: disable=too-many-locals
             model_name = device_info.get("model_name", model_name)
             manufacturer = device_info.get("manufacturer", manufacturer)
             udn = device_info.get("ssdp_udn", None)
+        else:
+            udn = status.get("ssdp_udn", None)
 
         if not display_supported:
             cast_type = CAST_TYPE_AUDIO
@@ -225,7 +250,8 @@ def get_device_info(  # pylint: disable=too-many-locals
         return None
 
 
-def _get_group_info(host, group):
+def _get_group_info(host: str, group: Any) -> MultizoneInfo:
+    """Parse group JSON data and return a MultizoneInfo instance."""
     name = group.get("name", "Unknown group name")
     udn = group.get("uuid", None)
     uuid = None
@@ -246,17 +272,18 @@ def _get_group_info(host, group):
     return MultizoneInfo(name, uuid, leader_host, leader_port)
 
 
-def get_multizone_status(host, services=None, zconf=None, timeout=30, context=None):
-    """
-    :param host: Hostname or ip to fetch status from
-    :type host: str
-    :return: The multizone status as a named tuple.
-    :rtype: pychromecast.dial.MultizoneStatus or None
-    """
+def get_multizone_status(
+    host: str,
+    services: set[HostServiceInfo | MDNSServiceInfo] | None = None,
+    zconf: zeroconf.Zeroconf | None = None,
+    timeout: float = 30,
+    context: ssl.SSLContext | None = None,
+) -> MultizoneStatus | None:
+    """Return a filled in MultizoneStatus object for the specified device."""
 
     try:
         if services is None:
-            services = [ServiceInfo(SERVICE_TYPE_HOST, (host, 8009))]
+            services = {HostServiceInfo(host, 8009)}
         _, status = _get_status(
             services,
             zconf,
@@ -282,18 +309,31 @@ def get_multizone_status(host, services=None, zconf=None, timeout=30, context=No
         return None
 
 
-MultizoneInfo = namedtuple("MultizoneInfo", ["friendly_name", "uuid", "host", "port"])
+@dataclass(frozen=True)
+class MultizoneInfo:
+    """Multizone info container."""
 
-MultizoneStatus = namedtuple("MultizoneStatus", ["dynamic_groups", "groups"])
+    friendly_name: str
+    uuid: UUID | None
+    host: str | None
+    port: int | None
 
-DeviceStatus = namedtuple(
-    "DeviceStatus",
-    [
-        "friendly_name",
-        "model_name",
-        "manufacturer",
-        "uuid",
-        "cast_type",
-        "multizone_supported",
-    ],
-)
+
+@dataclass(frozen=True)
+class MultizoneStatus:
+    """Multizone status container."""
+
+    dynamic_groups: list[MultizoneInfo]
+    groups: list[MultizoneInfo]
+
+
+@dataclass(frozen=True)
+class DeviceStatus:
+    """Device status container."""
+
+    friendly_name: str
+    model_name: str
+    manufacturer: str
+    uuid: UUID | None
+    cast_type: str
+    multizone_supported: bool
